///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// HHI
// K. Mueller
//
// Mitsubishi Electric - Infomation Technology Europe - Visual Information Laboratory
// James Cooper
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties
// from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 2000.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  MultiViewCS.h
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __MultiViewCS__
#define __MultiViewCS__

#include "Descriptors.h"
#include "Encoders.h"
#include "Decoders.h"

namespace XM
{

  //sth delimiters :-(
//#===---------------------------------
//#===--- MultiViewCSInterfaceABC

class MultiViewCSInterfaceABC: public I_InterfaceABC
{
public:
	MultiViewCSInterfaceABC();
	virtual ~MultiViewCSInterfaceABC() {}

	virtual void destroy()=0;

	static const UUID myID;

	virtual EncoderFileIO *GetEncoderStreamBuffer() = 0;
	virtual int SetEncoderStreamBuffer(EncoderFileIO *pBuffer) = 0;
	virtual DecoderFileIO *GetDecoderStreamBuffer() = 0;
	virtual int SetDecoderStreamBuffer(DecoderFileIO *pBuffer) = 0;

	virtual MultiViewDescriptorInterfaceABC *GetDescriptorInterface(void) = 0;
	virtual int SetDescriptorInterface(MultiViewDescriptorInterfaceABC
				    *pMultiViewDescriptorInterface) = 0;

	virtual SUB_CS_INTERFACE_ABC* Get2DCSInterface() = 0;
	virtual void Set2DCSInterface( SUB_CS_INTERFACE_ABC* a2DCSInterface) = 0;

	virtual int StartEncode() = 0;
	virtual int StartDecode() = 0;

};

//#===------------------------------
//#===--- MultiViewCSInterface
class MultiViewCS;

class MultiViewCSInterface: public MultiViewCSInterfaceABC
{
public:
	MultiViewCSInterface(MultiViewCS *pCS);
	virtual ~MultiViewCSInterface();

	virtual void destroy();
	
	static const UUID myID;
	static const char *myName;

	virtual const UUID& GetInterfaceID(void);
	virtual const char *GetName(void);

	virtual EncoderFileIO *GetEncoderStreamBuffer();
	virtual int SetEncoderStreamBuffer(EncoderFileIO *pBuffer);
	virtual DecoderFileIO *GetDecoderStreamBuffer();
	virtual int SetDecoderStreamBuffer(DecoderFileIO *pBuffer);

	virtual MultiViewDescriptorInterfaceABC *GetDescriptorInterface(void);
	virtual int SetDescriptorInterface(MultiViewDescriptorInterfaceABC
				    *pMultiViewDescriptorInterface);

		// accessor methods for subdescriptor
	virtual SUB_CS_INTERFACE_ABC* Get2DCSInterface();
	virtual void Set2DCSInterface( SUB_CS_INTERFACE_ABC* a2DCSInterface);

	virtual int StartEncode();
	virtual int StartDecode();

private:
	MultiViewCS *m_pMultiViewCS;	
};

//#===---------------------
//#===--- MultiViewCS

class MultiViewCS: public DescriptionCodingEngine
{
friend MultiViewCSInterface;
public:
	MultiViewCS();
	MultiViewCS(MultiViewDescriptorInterfaceABC *MultiView);

	// actual descriptor methods
	virtual bool IsProprietary(void);
	virtual const UUID& GetValueID(void);
	virtual const char* GetValueName(void);

	virtual const UUID& GetObjectID(void);
	virtual const char *GetName(void);

	// access is allowed only by class factories for this
	// object.  This avoids having to duplicate the
	// ID definition in multiple locations.  In the future, we may
	// have to do this.  PLEASE DON'T USE THESE UNLESS YOU ARE 
	// IMPLEMENTING A CLASS FACTORY GENERATING THIS OBJECT
	static const UUID myID;
	static const char * myName;

	virtual MultiViewCSInterfaceABC *GetInterface(void);

	// accessor methods
	virtual EncoderFileIO *GetEncoderStreamBuffer();
	virtual int SetEncoderStreamBuffer(EncoderFileIO *pBuffer);
	virtual DecoderFileIO *GetDecoderStreamBuffer();
	virtual int SetDecoderStreamBuffer(DecoderFileIO *pBuffer);

	virtual MultiViewDescriptorInterfaceABC *GetDescriptorInterface(void);
	virtual int SetDescriptorInterface(MultiViewDescriptorInterfaceABC
				    *pMultiViewDescriptorInterface);

		// accessor methods for subdescriptor
	virtual SUB_CS_INTERFACE_ABC* Get2DCSInterface();
	virtual void Set2DCSInterface( SUB_CS_INTERFACE_ABC* a2DCSInterface );

	virtual int StartEncode();
	virtual int StartDecode();

private:

	virtual ~MultiViewCS();

	MultiViewCSInterface m_Interface;
	
	const bool m_isProprietary;
	static const char * valName;
	static const UUID valID;

	// descriptor data
	EncoderFileIO *m_EncoderBuffer;
	DecoderFileIO *m_DecoderBuffer;
	MultiViewDescriptorInterfaceABC *m_pMultiViewDescriptorInterface;

	SUB_CS_INTERFACE_ABC* m_pShape2DCSInterface;
};

};

#endif //__MultiViewCS__
