///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// K.Masukura (R&D Center Toshiba, Japan)
// (contributing organizations names)
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties
// from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-2001.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  SpatioTemporalLocatorCS.h
#ifndef __SpatioTemporalLocatorCS__
#define __SpatioTemporalLocatorCS__

#include "Descriptors.h"
#include "Encoders.h"
#include "Decoders.h"

namespace XM
{


//=============================================================================
class SpatioTemporalLocatorCSInterfaceABC: public I_InterfaceABC
{
public:
	SpatioTemporalLocatorCSInterfaceABC();
	virtual ~SpatioTemporalLocatorCSInterfaceABC() {}

	virtual void destroy()=0;

	static const UUID myID;

	virtual EncoderFileIO *GetEncoderStreamBuffer(void) = 0;
	virtual int SetEncoderStreamBuffer(EncoderFileIO *aBuffer) = 0;
	virtual DecoderFileIO *GetDecoderStreamBuffer(void) = 0;
	virtual int SetDecoderStreamBuffer(DecoderFileIO *aBuffer) = 0;

	virtual SpatioTemporalLocatorDescriptorInterfaceABC
	  *GetDescriptorInterface(void) = 0;
	virtual int SetDescriptorInterface(SpatioTemporalLocatorDescriptorInterfaceABC
				    *aSpatioTemporalLocatorDescriptorInterface) = 0;


	virtual RegionLocatorCSInterfaceABC
	  *GetRegionLocatorCSInterface(void) = 0;
	virtual int SetRegionLocatorCSInterface(RegionLocatorCSInterfaceABC
					    *aRegionLocatorCSInterface) = 0;

	virtual TemporalInterpolationCSInterfaceABC
	  *GetTemporalInterpolationCSInterface(void) = 0;
	virtual int SetTemporalInterpolationCSInterface(
	    TemporalInterpolationCSInterfaceABC
	    *aTemporalInterpolationBCSInterface) = 0;

	virtual int StartEncode() = 0;
	virtual int StartDecode() = 0;

};

class SpatioTemporalLocatorCS;

//=============================================================================
class SpatioTemporalLocatorCSInterface: public SpatioTemporalLocatorCSInterfaceABC
{
public:
	SpatioTemporalLocatorCSInterface(SpatioTemporalLocatorCS *anCS);
	virtual ~SpatioTemporalLocatorCSInterface();
	
	static const UUID myID;
	static const char *myName;

	virtual void destroy();

	virtual const UUID& GetInterfaceID(void);
	virtual const char *GetName(void);

	virtual EncoderFileIO *GetEncoderStreamBuffer(void);
	virtual int SetEncoderStreamBuffer(EncoderFileIO *aBuffer);
	virtual DecoderFileIO *GetDecoderStreamBuffer(void);
	virtual int SetDecoderStreamBuffer(DecoderFileIO *aBuffer);

	virtual SpatioTemporalLocatorDescriptorInterfaceABC *GetDescriptorInterface(void);
	virtual int SetDescriptorInterface(SpatioTemporalLocatorDescriptorInterfaceABC
				    *aSpatioTemporalLocatorDescriptorInterface);

	virtual RegionLocatorCSInterfaceABC
	  *GetRegionLocatorCSInterface(void) ;
	virtual int SetRegionLocatorCSInterface(RegionLocatorCSInterfaceABC
						*aRegionLocatorCSInterface) ;

	virtual TemporalInterpolationCSInterfaceABC
	  *GetTemporalInterpolationCSInterface(void) ;
	virtual int SetTemporalInterpolationCSInterface(
          TemporalInterpolationCSInterfaceABC
	  *aTemporalInterpolationBCSInterface) ;

	virtual int StartEncode();
	virtual int StartDecode();

private:
	SpatioTemporalLocatorCS *m_SpatioTemporalLocatorCS;	
};

class EncoderFileIO;
class SpatioTemporalLocatorDescriptor;

//=============================================================================
class SpatioTemporalLocatorCS: public DescriptionCodingEngine
{
friend SpatioTemporalLocatorCSInterface;
public:
	SpatioTemporalLocatorCS();
	SpatioTemporalLocatorCS(SpatioTemporalLocatorDescriptorInterfaceABC
				*SpatioTemporalLocatorcs);


	// actual descriptor methods
	virtual bool IsProprietary(void);
	virtual const UUID& GetValueID(void);
	virtual const char* GetValueName(void);

	virtual const UUID& GetObjectID(void);
	virtual const char *GetName(void);

	// access is allowed only by class factories for this
	// object.  This avoids having to duplicate the
	// ID definition in multiple locations.  In the future, we may
	// have to do this.  PLEASE DON'T USE THESE UNLESS YOU ARE 
	// IMPLEMENTING A CLASS FACTORY GENERATING THIS OBJECT
	static const UUID myID;
	static const char * myName;

	virtual SpatioTemporalLocatorCSInterfaceABC *GetInterface(void);

	// accessor methods

	virtual EncoderFileIO *GetEncoderStreamBuffer(void);
	virtual int SetEncoderStreamBuffer(EncoderFileIO *aBuffer);
	virtual DecoderFileIO *GetDecoderStreamBuffer(void);
	virtual int SetDecoderStreamBuffer(DecoderFileIO *aBuffer);

	virtual SpatioTemporalLocatorDescriptorInterfaceABC
	  *GetDescriptorInterface(void);
	virtual int SetDescriptorInterface(
          SpatioTemporalLocatorDescriptorInterfaceABC
	  *aSpatioTemporalLocatorDescriptorInterface);

	virtual RegionLocatorCSInterfaceABC
	  *GetRegionLocatorCSInterface(void) ;
	virtual int SetRegionLocatorCSInterface(RegionLocatorCSInterfaceABC
						*aRegionLocatorCSInterface) ;

	virtual TemporalInterpolationCSInterfaceABC
	  *GetTemporalInterpolationCSInterface(void) ;
	virtual int SetTemporalInterpolationCSInterface(
          TemporalInterpolationCSInterfaceABC
	  *aTemporalInterpolationBCSInterface) ;

	virtual int StartEncode();
	virtual int StartDecode();

private:
	virtual ~SpatioTemporalLocatorCS(); 

	SpatioTemporalLocatorCSInterface m_Interface;
	
	const bool m_isProprietary;
	static const char * valName;
	static const UUID valID;

	// descriptor data
	EncoderFileIO *m_EncoderBuffer;
	DecoderFileIO *m_DecoderBuffer;

	SpatioTemporalLocatorDescriptorInterfaceABC
	  *m_SpatioTemporalLocatorDescriptorInterface;

	RegionLocatorCSInterfaceABC *m_RegionLocatorCSInterface;
	TemporalInterpolationCSInterfaceABC 
	  *m_TemporalInterpolationCSInterface;

	unsigned long GetVluimsbf();
	unsigned long PutVluimsbf(unsigned long value);
	unsigned long EncodeTimePoint(double t);
	unsigned long EncodeDuration(double t);
	double DecodeTimePoint();
	double DecodeDuration();

};

};

#endif //__SpatioTemporalLocatorCS__
