///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// University of California Santa Barbara, Kapil Chhabra, Yining Deng
// Mitsubishi Electric ITE-VIL, Leszek Cieplinski, LG-Elite.
// (contributing organizations names)
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties
// from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-2000.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DominantColorExtraction.h
//
#ifndef __DOMINANTCOLOREXTRACTION_H__
#define __DOMINANTCOLOREXTRACTION_H__
#include "XMInterfaces.h"                                   /* Naming Convention */
#include "XMObjects.h"                                      /* Naming Convention */
#include <momusys.h>
#include <intra.h>
#include "imgutil.h"
#include "Media.h"

//=============================================================================

namespace XM
{


// Forward Declarations:
class DominantColorDescriptor;
class DominantColorExtractionTool;

//=============================================================================
class DominantColorExtractionInterfaceABC: public I_InterfaceABC
{
public:
  DominantColorExtractionInterfaceABC();
  virtual ~DominantColorExtractionInterfaceABC() {};

  virtual void destroy()=0;

  virtual int SetSourceMedia(MultiMediaInterfaceABC *media) = 0;
  virtual DominantColorDescriptorInterfaceABC* GetDescriptorInterface()=0;
  virtual int SetDescriptorInterface(DominantColorDescriptorInterfaceABC
				     *aDominantColorDescriptorInterface)=0;
  virtual unsigned long StartExtracting()=0; 
                                                                     
  static const UUID myID;
};

//=============================================================================
class DominantColorExtractionInterface:
    public DominantColorExtractionInterfaceABC
{
public:

  DominantColorExtractionInterface(DominantColorExtractionTool* aTool);
  virtual ~DominantColorExtractionInterface();

  virtual const UUID& GetInterfaceID(void);
  virtual const char *GetName(void);

  virtual void destroy();

  virtual int SetSourceMedia(MultiMediaInterfaceABC *media);
  virtual DominantColorDescriptorInterfaceABC* GetDescriptorInterface();
  virtual int SetDescriptorInterface(DominantColorDescriptorInterfaceABC
				     *aDominantColorDescriptorInterface);
  virtual unsigned long StartExtracting();

  static const UUID myID;
  static const char *myName;

private:

  DominantColorExtractionTool *m_ExtractionTool;

};

//=============================================================================
class DominantColorExtractionTool: public DescriptorExtractor
{
  friend DominantColorExtractionInterface;

public:

  DominantColorExtractionTool();
  DominantColorExtractionTool(DominantColorDescriptorInterfaceABC *dc);

  virtual const UUID& GetObjectID(void);
  virtual const char *GetName(void);

  virtual bool SupportsPush(void);
  virtual bool SupportsPull(void);

  virtual int SetSourceMedia(MultiMediaInterfaceABC *media);
  virtual DominantColorDescriptorInterfaceABC* GetDescriptorInterface();
  virtual int SetDescriptorInterface(DominantColorDescriptorInterfaceABC
                                     *aDominantColorDescriptorInterface);
  virtual unsigned long StartExtracting();

  virtual DominantColorExtractionInterfaceABC *GetInterface(void);

  // access is allowed only by class factories for this
  // object.  This avoids having to duplicate the
  // ID definition in multiple locations.  In the future, we may
  // have to do this.  PLEASE DO NOT USE THESE UNLESS YOU ARE 
  // IMPLEMENTING A CLASS FACTORY GENERATING THIS OBJECT
  static const UUID myID;
  static const char * myName;

private:

  void Extract( float *imdata, int imsize, unsigned char *quantImageAlpha,
		MomVop *ImageMedia );
  double Cluster( int *closest, float *imdata, int imsize,
		  unsigned char *quantImageAlpha );
  void Centroids( int *closest, float *imdata, int imsize,
		  unsigned char *quantImageAlpha );
  double Dist( int *closest, float *imdata, int imsize,
	       unsigned char *quantImageAlpha );
  void Vars( int *closest, float *imdata, int imsize,
	     unsigned char *quantImageAlpha );
  void Split( int *closest, float *imdata, int imsize, double factor,
	      unsigned char *quantImageAlpha );
  void Agglom( double distthr );
  
  int QuantizeSC(double sc);
  int GetSpatialCoherency(float *ColorData, int dim, int N, float **col_float,
			  unsigned char *quantImageAlpha, MomVop *ImageMedia);
  double GetCoherencyWithColorAllow(float *ColorData, int dim, bool *IVisit,
				    float l, float u, float v, float Allow,
				    int NeighborRange,
				    unsigned int *OUTPUT_Corres_Pixel_Count,
				    MomVop *ImageMedia);

  virtual ~DominantColorExtractionTool();

  DominantColorExtractionInterface m_Interface;
  DominantColorDescriptorInterfaceABC *m_DescriptorInterface;
  MultiMediaInterfaceABC *m_Media;

  int     m_MaxSize;
  int     m_CurrSize;
  float  *m_Weights;
  float **m_Centroids;
  float **m_Variances;
  float   m_Coherency;

};

};


#endif //__DOMINANTCOLOREXTRACTION_H__
