///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// K.Masukura (R&D Center Toshiba, Japan)
// (contributing organizations names)
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties
// from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-2001.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  TemporalInterpolationExtraction.h
//
#ifndef __TemporalInterpolationEXTRACTION_H__
#define __TemporalInterpolationEXTRACTION_H__
#include "XMInterfaces.h"                                   /* Naming Convention */
#include "XMObjects.h"                                      /* Naming Convention */
#include "Media.h"

#include <deque>
#include "temporal_interpolation_extraction.h"

// Define of Structs
struct SIPoint{
	double PointT;
	double Point[16];
};

struct SInputPoints {	// Input Point list
	int					Npoint;
	int					Dimension;
	std::deque<SIPoint>	Point;
};



//=============================================================================
//Macro for extraction process

namespace XM
{


// Forward Declarations:
class TemporalInterpolationExtractionTool;


//=============================================================================
class TemporalInterpolationExtractionInterfaceABC: public I_InterfaceABC
{
public:
	TemporalInterpolationExtractionInterfaceABC();
	virtual ~TemporalInterpolationExtractionInterfaceABC() {};

	virtual void destroy()=0;

	virtual int SetSourceMedia(MultiMediaInterfaceABC *media) =0;

	virtual TemporalInterpolationDescriptorInterfaceABC
	  *GetDescriptorInterface(void) =0;
	virtual int SetDescriptorInterface(TemporalInterpolationDescriptorInterfaceABC
				    *aTemporalInterpolationDescriptorInterface) = 0;
	virtual unsigned long StartExtracting(void) = 0;
	virtual unsigned long PostExtraction(void) = 0;

	virtual TemporalInterpolationExtractionTool* ExTool()=0;

	static const UUID myID;
};

//=============================================================================
class TemporalInterpolationExtractionInterface: 
  public TemporalInterpolationExtractionInterfaceABC
{
public:
	TemporalInterpolationExtractionInterface(TemporalInterpolationExtractionTool* aTool);
	virtual ~TemporalInterpolationExtractionInterface();
	
	virtual void destroy();

	virtual const UUID& GetInterfaceID(void);
	virtual const char *GetName(void);

	virtual int SetSourceMedia(MultiMediaInterfaceABC *media);

	virtual TemporalInterpolationDescriptorInterfaceABC
	  *GetDescriptorInterface(void);
	virtual int SetDescriptorInterface(TemporalInterpolationDescriptorInterfaceABC
					  *aTemporalInterpolationDescriptorInterface);
	virtual unsigned long StartExtracting(void);
	virtual unsigned long PostExtraction(void);

	static const UUID myID;
	static const char *myName;

	virtual TemporalInterpolationExtractionTool* ExTool();

private:
	TemporalInterpolationExtractionTool *m_ExtractionTool;
};

//=============================================================================
class TemporalInterpolationExtractionTool: public DescriptorExtractor
{
friend TemporalInterpolationExtractionInterface;
public:
	TemporalInterpolationExtractionTool();
	TemporalInterpolationExtractionTool(TemporalInterpolationDescriptorInterfaceABC *TemporalInterpolation);

	virtual const UUID& GetObjectID(void);
	virtual const char *GetName(void);

	virtual bool SupportsPush(void);
	virtual bool SupportsPull(void);

	virtual int SetSourceMedia(MultiMediaInterfaceABC *media);

	virtual TemporalInterpolationDescriptorInterfaceABC
	  *GetDescriptorInterface(void);
	virtual int SetDescriptorInterface(TemporalInterpolationDescriptorInterfaceABC
				   * aTemporalInterpolationDescriptorInterface);
	
	virtual unsigned long StartExtracting(void);
	virtual unsigned long PostExtraction(void);

	virtual TemporalInterpolationExtractionInterfaceABC *GetInterface(void);


	// access is allowed only by class factories for this
	// object.  This avoids having to duplicate the
	// ID definition in multiple locations.  In the future, we may
	// have to do this.  PLEASE DO NOT USE THESE UNLESS YOU ARE 
	// IMPLEMENTING A CLASS FACTORY GENERATING THIS OBJECT
	static const UUID myID;
	static const char * myName;
private:
	virtual ~TemporalInterpolationExtractionTool(); 

	TemporalInterpolationExtractionInterface m_Interface;
	TemporalInterpolationDescriptorInterfaceABC *m_DescriptorInterface;


	MultiMediaInterfaceABC *m_Media;
	CInterp Interp;
	int m_counter;

	SInputPoints InputPoints;	

	// for CInterp
	int pDeg[16];
	double pErr[16];
	char *m_pMode;
	double m_Interval;
	int pUseDefaultFunction[16];
};

};


#endif //__TemporalInterpolationEXTRACTION_H__
