/***********************************************************************
This software module was originally developed by Cdric Thinot (Expway)
Claude Seyrat (Expway) and Grgoire Pau (Expway) in the course of 
development of the MPEG-7 Systems (ISO/IEC 15938-1) standard. 

This software module is an implementation of a part of one or more 
MPEG-7 Systems (ISO/IEC 15938-1) tools as specified by the 
MPEG-7 Systems (ISO/IEC 15938-1) standard. 

ISO/IEC gives users of the MPEG-7 Systems (ISO/IEC 15938-1) free license
to this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-7 Systems 
(ISO/IEC 15938-1). 

Those intending to use this software module in hardware or software 
products are advised that its use may infringe existing patents.

The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. 

Copyright is not released for non MPEG-7 Systems (ISO/IEC 15938-1) 
conforming products. 

Expway retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-7 Systems (ISO/IEC 15938-1) conforming 
products. 

This copyright notice must be included in all copies or derivative works.

Copyright Expway  2001.
************************************************************************/

package com.expway.tools.compression;

import com.expway.tools.automata.*;
import com.expway.tools.expression.*;
import java.util.Collection;

/** refuse une transition si l'activit (qui doit tre une CompressionActivityToken)
    a un compteur plus grand que prvu */

abstract public class LengthConditionalTransition extends LengthTransition {
    
    public LengthConditionalTransition(String accept, State fromS, State toS){
        super(accept,fromS,toS);
    }
    
    protected int min=0, max=OccurrenceNode.UNBOUNDED;
    
    /** met en place une transition conditionnelle, la valeur retourne
        par l'activit doit etre comprise entre min et max */ 
    public void setLengthTransitionConditions(int min,int max){
        this.min=min;
        this.max=max;
    }
    
    /** 
     * appel pour savoir si la conjonction vnement et activit est 
     * susceptible d'tre accepte par la transition le token ne doit
     * pas tre modifi par la mthode 
     */
    
    public boolean epsilonAccept(ActivityToken at,Object applicationContext){
        //System.out.println("EPSILONACCEPT of " + this + " on cat " + ((CompressionActivityToken)at).name() );
        //((CompressionActivityToken)at).log(this + " (EPSILONACCEPT)");
        int l = ((CompressionActivityToken)at).getCpt();
        if (l<min) {
            //System.out.println(" refused because length("+l+") < min ("+min+")");
            return false;
        }
        else if (l>=max && max!=OccurrenceNode.UNBOUNDED){
            //System.out.println(" refused because  length("+l+") >= max ("+max+")");
            return false;
        }
        return true;
    }
    
    /*public String getLabel(){
        return super.getLabel() + ", " + min + "< Cpt <" + (max==OccurrenceNode.UNBOUNDED?"*":""+max);
        }*/

}
