/* 
 * (C) Lehrstuhl fuer Integrierte Schaltungen, TU Muenchen
 */

/******* SCCSInfo ************************************************************/
/* Filename :    $RCSfile: readparam.c,v $
   Version  :    $Revision: 1.6 $
   Last Edit:    $Date: 2001/11/12 16:51:20 $ 
   Released :    %D% %T% */
/******* SCCSInfo ************************************************************/

///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// S. Herrmann TU-Munich, Institute for Integrated Circuits
// (contributing organizations names)
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties
// from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  readparam.c
//

/**********************************************************************/
/*  INCLUDES                                                          */
/**********************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#ifndef _VISUALC_
#include<unistd.h>
#else
#include<io.h>
#endif
#include"readparam.h"

/**********************************************************************/
/*  PROTOTYPES                                                        */
/**********************************************************************/
#ifdef ANSI

static void memerror();

#else

static void memerror();

#endif

struct param_struct {
  char *name;
  char *value;
  struct param_struct *next;
};

typedef struct param_struct param;

static param *firstpara=(param *)0;


/**********************************************************************/
/*  FUNCTIONS                                                         */
/**********************************************************************/


/**********************************************************************/
/*  readparameters                                                    
    this procedure opens the parfilename and copies the
    contens to the buffer parabuffer.
    than the funktions to read specific parameters are called*/
/**********************************************************************/
void readparameters(char *parfilename)

/*char *parfilename;*/

{
  FILE *parafile;
  char string[512];
  int comment, name;
  param **newparam;
  param *nextparam;
  int i;
  char *p;
  
#ifdef _VISUALC_ //sth to fix compiler bug in VC++ 6
  if (firstpara < (param *)0xff) firstpara=NULL;
#endif
  
  parafile=fopen(parfilename,"rb");
  if (!parafile) {
    if (strlen(parfilename) < 471) {
      sprintf(string,"ERROR : Could not open parameterfile (%s)!",
	      parfilename);
    }
    else {
      sprintf(string,"ERROR : Could not open parameterfile (name to long)!");
    }
    perror(string);
    exit(2);
  }

  i=0;
  name=0;
  comment=0;
  nextparam=firstpara;
  newparam=&firstpara;
  while (fread(&string[i],1,1,parafile)) {
    if (i) {                            /* start of comment*/
      if ((string[i-1]== '/') && (string[i]=='*')) {
	i--;
	comment=1;
	continue;
      }
    }

    if (comment) {                     /* end of comment*/
      while (string[i]== '*') {
	fread(&string[i],1,1,parafile);
	if (string[i]== '/') {
	  comment=0;
	}
      }
      if (!comment) continue;
    }
    else {
      if (strchr(" \t\n\x0",string[i])) {    /* if delimiter*/
	if (!i) {
	  if (name && (string[0]=='\n')) {
	    name=0;
	    newparam=&((*newparam)->next);
	  }
	  continue;                          /* starting space or end of line*/
	}
	if (!name) {                         /* end of name*/  
	  *newparam=(param *)malloc(sizeof(param));
	  if (!*newparam) memerror();
	  (*newparam)->name=(char *)malloc(i+1);
	  if (!(*newparam)->name) memerror();
	  (*newparam)->value=0;
	  (*newparam)->next=nextparam;
	  if (string[i] != '\n') name=1;
	  string[i]=0;
	  while ((p=strrchr(string,'\n'))||(p=strrchr(string,'\r'))) *p=0;
	  strcpy((*newparam)->name,string);
	  i=0;
	  if (!name) newparam=&((*newparam)->next);
	  continue;
	}
	if (string[i] == '\n') {
	  (*newparam)->value=(char *)malloc(i+1);
	  if (!(*newparam)->value) memerror();
	  string[i]=0;
	  while ((p=strrchr(string,'\n'))||(p=strrchr(string,'\r'))) *p=0;
	  strcpy((*newparam)->value,string);
	  i=0;
	  name=0;
	  newparam=&((*newparam)->next);
	  continue;
	}
	
	if (string[i-1] == ' ') {     /* more the one white space*/
	  continue;
	}
	string[i]= ' ';
	i++;
	continue;
      }
      i++;
    }
  }

  if (i) {                          /* file not terminatetd with \n*/
    if (!name) {
      *newparam=(param *)malloc(sizeof(param));
      if (!*newparam) memerror();
      i++;
      (*newparam)->name=(char *)malloc(i+1);
      if (!(*newparam)->name) memerror();
      (*newparam)->value=0;
      (*newparam)->next=0;
      string[i]=0;
      strcpy((*newparam)->name,string);
    }
    else {
      i++;
      (*newparam)->value=(char *)malloc(i+1);
      if (!(*newparam)->value) memerror();
      string[i]=0;
      strcpy((*newparam)->value,string);
    }
  }
  fclose(parafile);
}

/**********************************************************************/
/* memerror   prints errormessage                                     */
/**********************************************************************/

static void memerror()

{
  fprintf(stderr,"ERROR : Out of memory!\n");
  exit (1);
}


/**********************************************************************/
/*  getpara
    searches for a parameter in the parameterbuffer and
    returns the value
    if not found, return default-value*/
/**********************************************************************/
char *getpara(char *paraname,char * deflt)

/*char *param;
char *deflt;*/

{
  param *para;

  para=firstpara;

  while (para){
    if (!strcmp(para->name,paraname)) {
      if (!para->value) return "";
      return para->value;
    }
    para=para->next;
  }
  return(deflt);
}

/**********************************************************************/
/*  testpara
    if parameter param is found in parfilename the
    returnvalue is 1 els it is 0*/
/**********************************************************************/
int testpara(char *paraname)

/*char *param;
char *deflt;*/

{
  param *para;

  para=firstpara;

  while (para){
    if (!strcmp(para->name,paraname)) return 1;
    para=para->next;
  }
  return(0);
}


/**********************************************************************/
/*  strtrunc
    deletes trailing " \t\n" from string*/
/**********************************************************************/
char *strtrunc(char *string)

/*char *string;*/

{
  char *a;
  
  a=strchr(string,'\n');
  if (a) *a=0;
  a=string;
  a+=strlen(a);
  a--;
  
  while (strchr(" \t\n",*a) && (a>=string)) {
    *a=0;
    a--;
  }
  return(a);
}

